@@ -21,8 +21,9 @@ import qualified Data.EventList.Relative.TimeBodyPrivate as TimeBodyPriv
 
 import Data.EventList.Relative.MixedBody ((/.), (./), empty)
 
+import qualified Numeric.NonNegative.Chunky as NonNegChunky
 import qualified Numeric.NonNegative.Class as NonNeg
-import Numeric.NonNegative.Class ((-|))
+import Numeric.NonNegative.Class ((-|), zero, add, )
 import Data.EventList.Relative.TimeBody (isNormalized)
 
 import Data.Tuple.HT (mapFst, mapPair, )
@@ -208,7 +209,7 @@ decreaseStartInfinite dif =
 delayAdditive :: (NonNeg.C time, Eq body) =>
    time -> time -> TimeBodyList.T time body -> Bool
 delayAdditive dif0 dif1 xs =
-   TimeBodyList.delay (dif0+dif1) xs ==
+   TimeBodyList.delay (add dif0 dif1) xs ==
    TimeBodyList.delay dif0 (TimeBodyList.delay dif1 xs)
 
 delayAppendPause :: (NonNeg.C time, Eq body) =>
@@ -234,9 +235,9 @@ splitAtTakeDropTime t xs =
 takeTimeEndPause :: (NonNeg.C time, Ord body) =>
    time -> TimeBodyList.T time body -> Bool
 takeTimeEndPause t xs =
-   t == 0 ||
+   t == zero ||
    t >= TimeBodyList.duration xs ||
-   0 <  snd (TimeMixedList.viewTimeR (TimeMixedList.takeTime t xs))
+   zero <  snd (TimeMixedList.viewTimeR (TimeMixedList.takeTime t xs))
 
 takeTimeAppendFirst :: (NonNeg.C time, Eq body) =>
    time -> TimeBodyList.T time body -> TimeBodyList.T time body -> Bool
@@ -246,12 +247,12 @@ takeTimeAppendFirst t xs ys =
       (TimeMixedList.takeTime t xs)
       (TimeMixedList.takeTime (t -| TimeBodyList.duration xs) ys)
 
-takeTimeAppendSecond :: (NonNeg.C time, Eq body) =>
+takeTimeAppendSecond :: (NonNeg.C time, Num time, Eq body) =>
    time -> TimeBodyList.T time body -> TimeBodyList.T time body -> Bool
 takeTimeAppendSecond t xs ys0 =
    -- the second list must not start with a zero pause
    let ys = TimeBodyList.delay 1 ys0
-       t1 = t+1
+       t1 = add t 1
    in  TimeMixedList.takeTime (TimeBodyList.duration xs + t1) (TimeBodyList.append xs ys) ==
        TimeMixedList.prependBodyEnd xs (TimeMixedList.takeTime t1 ys)
 
@@ -265,17 +266,34 @@ dropTimeNormalize :: (NonNeg.C time, Ord body) =>
 dropTimeNormalize t =
    isNormalized . TimeMixedList.dropTime t . TimeBodyList.normalize
 
-takeTimeInfinite :: (NonNeg.C time, Ord body) =>
+takeTimeInfinite :: (NonNeg.C time, Num time, Ord body) =>
    time -> NonEmptyList time body -> Bool
 takeTimeInfinite t =
    (t == ) . TimeTimeList.duration .
    TimeMixedList.takeTime t . makeUncollapsedInfiniteEventList
 
-dropTimeInfinite :: (NonNeg.C time, Ord body) =>
+dropTimeInfinite :: (NonNeg.C time, Num time, Ord body) =>
    time -> NonEmptyList time body -> Bool
 dropTimeInfinite t =
    checkInfinite . TimeMixedList.dropTime t . makeUncollapsedInfiniteEventList
 
+_splitAtTimeLazyInfinite ::
+   (NonNeg.C time, Num time, Ord body, Show time, Show body) =>
+   BodyBodyList.T (NonNegChunky.T time) body -> Bool
+_splitAtTimeLazyInfinite =
+   not . null . show . snd .
+   TimeMixedList.splitAtTime 1000000 .
+   MixedBodyList.consTime
+      (NonNegChunky.fromChunks $ iterate (2-) 1)
+
+_dropTimeLazyInfinite ::
+   (NonNeg.C time, Num time, Ord body, Show time, Show body) =>
+   BodyBodyList.T (NonNegChunky.T time) body -> Bool
+_dropTimeLazyInfinite =
+   not . null . show .
+   TimeMixedList.dropTime 1000000 .
+   MixedBodyList.consTime
+      (NonNegChunky.fromChunks $ iterate (2-) 1)
 
 
 
@@ -284,7 +302,7 @@ durationPause :: (NonNeg.C time) =>
 durationPause t =
    t == TimeBodyList.duration (TimeBodyList.singleton t (error "durationPause: no need to access body"))
 
-durationAppend :: (NonNeg.C time) =>
+durationAppend :: (NonNeg.C time, Num time) =>
    TimeBodyList.T time body -> TimeBodyList.T time body -> Bool
 durationAppend xs ys =
    TimeBodyList.duration (TimeBodyList.append xs ys)  ==
@@ -323,20 +341,20 @@ cycleInfinite xs0 =
    in  equalPrefix 100 xs (TimeBodyList.cycle xs)
 
 
-filterSatisfy :: (Num time) =>
+filterSatisfy :: (NonNeg.C time) =>
    (body -> Bool) ->
    TimeBodyList.T time body -> Bool
 filterSatisfy p =
    all p . TimeBodyList.getBodies . TimeBodyList.filter p
 
-filterProjection :: (Num time, Eq body) =>
+filterProjection :: (NonNeg.C time, Eq body) =>
    (body -> Bool) ->
    TimeBodyList.T time body -> Bool
 filterProjection p xs =
    TimeBodyList.filter p xs ==
    TimeBodyList.filter p (TimeBodyList.filter p xs)
 
-filterCommutative :: (Num time, Eq body) =>
+filterCommutative :: (NonNeg.C time, Eq body) =>
    (body -> Bool) ->
    (body -> Bool) ->
    TimeBodyList.T time body -> Bool
@@ -344,7 +362,7 @@ filterCommutative p q xs =
    TimeBodyList.filter p (TimeBodyList.filter q xs) ==
    TimeBodyList.filter q (TimeBodyList.filter p xs)
 
-filterComposition :: (Num time, Eq body) =>
+filterComposition :: (NonNeg.C time, Eq body) =>
    (body -> Bool) ->
    (body -> Bool) ->
    TimeBodyList.T time body -> Bool
@@ -503,6 +521,12 @@ insertCommutative (time0, body0) (time1, body1) evs =
    TimeBodyList.insert time0 body0 (TimeBodyList.insert time1 body1 evs)  ==
    TimeBodyList.insert time1 body1 (TimeBodyList.insert time0 body0 evs)
 
+{-
+Relative.BodyEnd.insert merge: Falsifiable, after 12 tests:
+2
+'}'
+1 /. '%' ./ 1 /. '}' ./ 0 /. ' ' ./ 5 /. 'z' ./ 5 /. '\'' ./ 2 /. '\DEL' ./ 2 /. 'x' ./ 3 /. '\DEL' ./ empty
+-}
 insertMerge :: (NonNeg.C time, Ord body) =>
    time -> body -> TimeBodyList.T time body -> Bool
 insertMerge time body evs =
@@ -523,7 +547,7 @@ insertSplitAtTime time body evs =
    ==
       let (prefix,suffix) = TimeMixedList.splitAtTime time evs
       in  TimeBodyList.normalize (TimeMixedList.appendBodyEnd prefix
-             (MixedBodyList.consTime 0 (MixedBodyList.consBody body suffix)))
+             (MixedBodyList.consTime zero (MixedBodyList.consBody body suffix)))
 
 insertInfinite :: (NonNeg.C time, Ord body) =>
    time -> body -> NonEmptyList time body -> Bool
@@ -532,21 +556,21 @@ insertInfinite time body =
 
 
 
-moveForwardIdentity :: (NonNeg.C time, Ord body) =>
+moveForwardIdentity :: (NonNeg.C time, Num time, Ord body) =>
    TimeBodyList.T time body -> Bool
 moveForwardIdentity evs =
    evs ==
-   TimeBodyList.moveForward (TimeBodyList.mapBody ((,) 0) evs)
+   TimeBodyList.moveForward (TimeBodyList.mapBody ((,) zero) evs)
 
-moveForwardAdditive :: (NonNeg.C time, Ord body) =>
+moveForwardAdditive :: (NonNeg.C time, Num time, Ord body) =>
    TimeBodyList.T time ((time,time),body) -> Bool
 moveForwardAdditive evs =
    TimeBodyList.normalize (moveForwardLimited (moveForwardLimited
       (TimeBodyList.mapBody (\((t0,t1),b) -> (t0,(t1,b))) evs))) ==
    TimeBodyList.normalize (moveForwardLimited
-      (TimeBodyList.mapBody (mapFst (uncurry (+))) evs))
+      (TimeBodyList.mapBody (mapFst (uncurry add)) evs))
 
-moveForwardCommutative :: (NonNeg.C time, Ord body) =>
+moveForwardCommutative :: (NonNeg.C time, Num time, Ord body) =>
    TimeBodyList.T time ((time,time),body) -> Bool
 moveForwardCommutative evs =
    TimeBodyList.normalize (moveForwardLimited (moveForwardLimited
@@ -575,16 +599,16 @@ moveForwardRestrictedInfinity maxTime =
 -}
 
 
-moveForwardLimited :: (NonNeg.C time) =>
+moveForwardLimited :: (NonNeg.C time, Num time) =>
    TimeBodyList.T time (time,body) -> TimeBodyList.T time body
 moveForwardLimited = TimeBodyList.moveForward . restrictMoveTimes
 
 restrictMoveTimes :: (NonNeg.C time) =>
    TimeBodyList.T time (time,body) -> TimeBodyList.T time (time,body)
 restrictMoveTimes =
-   flip evalState 0 .
+   flip evalState zero .
    TimeBodyList.mapM
-      (\t -> modify (t+) >> return t)
+      (\t -> modify (add t) >> return t)
       (\(t,b) -> gets (\tm -> (min t tm, b)))
 
 
@@ -619,14 +643,14 @@ collectCoincidentGaps :: (NonNeg.C time, Eq body) =>
    TimeBodyList.T time body -> Bool
 collectCoincidentGaps xs =
    let times = TimeBodyList.getTimes (TimeBodyList.collectCoincident xs)
-   in  null times || all (0<) (tail times)
+   in  null times || all (zero<) (tail times)
 
 collectCoincidentNonEmpty :: (NonNeg.C time, Eq body) =>
    TimeBodyList.T time body -> Bool
 collectCoincidentNonEmpty =
    all (not . null) . TimeBodyList.getBodies . TimeBodyList.collectCoincident
 
-collectCoincidentInfinite :: (NonNeg.C time, Eq body) =>
+collectCoincidentInfinite :: (NonNeg.C time, Num time, Eq body) =>
    NonEmptyList time body -> Bool
 collectCoincidentInfinite =
    checkInfinite .
@@ -720,7 +744,7 @@ propInsertPadded (Event time body) evs =
    EventList.insert time body (fst evs)  ==  fst (insert time body evs)
 -}
 
-appendSingle :: -- (Num time, Ord time, Ord body) =>
+appendSingle :: -- (NonNeg.C time, Ord time, Ord body) =>
    body -> TimeBodyList.T time body -> EventList.T time body
 appendSingle body xs =
    Disp.foldr EventList.consTime EventList.consBody EventList.empty $
@@ -742,7 +766,7 @@ toEventList xs =
 discretize :: (RealFrac time, Integral i) =>
    TimeBodyList.T time body -> TimeBodyList.T i body
 discretize es =
-   evalState (Uniform.mapSecondM roundDiff es) 0
+   evalState (Uniform.mapSecondM roundDiff es) zero
 
 resample :: (RealFrac time, Integral i) =>
    time -> TimeBodyList.T time body -> TimeBodyList.T i body
@@ -755,7 +779,7 @@ toAbsoluteEventList :: (Num time) =>
 toAbsoluteEventList start xs =
    let ts = Uniform.getSeconds xs
        bs = Uniform.getFirsts  xs
-       ats = List.scanl (+) start ts
+       ats = List.scanl add start ts
    in  maybe
           (error "padded list always contains one time value")
           (\ ~(ats0,lt) -> (zip ats0 bs, lt))
@@ -767,11 +791,12 @@ toAbsoluteEventList start xs =
 
 type NonEmptyList time body = (time, body, TimeBodyList.T time body)
 
-makeUncollapsedInfiniteEventList :: (NonNeg.C time) =>
+makeUncollapsedInfiniteEventList ::
+   (NonNeg.C time, Num time) =>
    NonEmptyList time body -> TimeBodyList.T time body
 makeUncollapsedInfiniteEventList =
    makeInfiniteEventList .
-   (\(time,body,xs) -> (time+1,body,xs))
+   (\(time,body,xs) -> (add time 1, body, xs))
 
 makeInfiniteEventList :: (NonNeg.C time) =>
    NonEmptyList time body -> TimeBodyList.T time body
@@ -826,17 +851,17 @@ tests =
      quickCheck (mapBodyComposition toUpper toLower
                :: TimeBodyList.T TimeDiff ArbChar -> Bool)) :
    ("map time composition",
-     quickCheck ((\dt0 dt1 -> mapTimeComposition (dt0+) (dt1+))
+     quickCheck ((\dt0 dt1 -> mapTimeComposition (add dt0) (add dt1))
                :: TimeDiff -> TimeDiff -> TimeBodyList.T TimeDiff ArbChar -> Bool)) :
    ("map time body commutative",
-     quickCheck ((\dt -> mapTimeBodyCommutative (dt+) toUpper)
+     quickCheck ((\dt -> mapTimeBodyCommutative (add dt) toUpper)
                :: TimeDiff -> TimeBodyList.T TimeDiff ArbChar -> Bool)) :
 
    ("mapBodyInfinite",
      quickCheck (mapBodyInfinite toUpper
                :: NonEmptyList TimeDiff ArbChar -> Bool)) :
    ("mapTimeInfinite",
-     quickCheck (\dt -> mapTimeInfinite (dt+)
+     quickCheck (\dt -> mapTimeInfinite (add dt)
                :: NonEmptyList TimeDiff ArbChar -> Bool)) :
 
    ("mapNormalize",
@@ -894,6 +919,12 @@ tests =
      quickCheck (takeTimeInfinite :: TimeDiff -> NonEmptyList TimeDiff ArbChar -> Bool)) :
    ("dropTimeInfinite",
      quickCheck (dropTimeInfinite :: TimeDiff -> NonEmptyList TimeDiff ArbChar -> Bool)) :
+{-
+   ("splitAtTimeLazyInfinite",
+     quickCheck (splitAtTimeLazyInfinite :: BodyBodyList.T (NonNegChunky.T TimeDiff) ArbChar -> Bool)) :
+   ("dropTimeLazyInfinite",
+     quickCheck (dropTimeLazyInfinite :: BodyBodyList.T (NonNegChunky.T TimeDiff) ArbChar -> Bool)) :
+-}
 
    ("duration pause",
      quickCheck (durationPause :: TimeDiff -> Bool)) :
